@@ -29,10 +29,11 @@
 # ===================================================================
 
 import unittest
+from binascii import unhexlify
 
 from Crypto.SelfTest.loader import load_tests
 from Crypto.SelfTest.st_common import list_test_cases
-from Crypto.Util.py3compat import tobytes, unhexlify, _memoryview
+from Crypto.Util.py3compat import tobytes, _memoryview, is_string
 from Crypto.Cipher import AES, DES3, DES
 from Crypto.Hash import SHAKE128
 
@@ -111,20 +112,20 @@ class BlockChainingTests(unittest.TestCase):
 
     def test_unaligned_data_128(self):
         cipher = AES.new(self.key_128, self.aes_mode, self.iv_128)
-        for wrong_length in xrange(1,16):
+        for wrong_length in range(1,16):
             self.assertRaises(ValueError, cipher.encrypt, b"5" * wrong_length)
 
         cipher = AES.new(self.key_128, self.aes_mode, self.iv_128)
-        for wrong_length in xrange(1,16):
+        for wrong_length in range(1,16):
             self.assertRaises(ValueError, cipher.decrypt, b"5" * wrong_length)
 
     def test_unaligned_data_64(self):
         cipher = DES3.new(self.key_192, self.des3_mode, self.iv_64)
-        for wrong_length in xrange(1,8):
+        for wrong_length in range(1,8):
             self.assertRaises(ValueError, cipher.encrypt, b"5" * wrong_length)
 
         cipher = DES3.new(self.key_192, self.des3_mode, self.iv_64)
-        for wrong_length in xrange(1,8):
+        for wrong_length in range(1,8):
             self.assertRaises(ValueError, cipher.decrypt, b"5" * wrong_length)
 
     def test_IV_iv_attributes(self):
@@ -230,10 +231,62 @@ class BlockChainingTests(unittest.TestCase):
         ref4 = cipher4.decrypt(data_mv)
 
         self.assertEqual(ref3, ref4)
+    
+    def test_output_param(self):
 
-    import types
-    if _memoryview is types.NoneType:
+        pt = b'5' * 16
+        cipher = AES.new(b'4'*16, self.aes_mode, iv=self.iv_128)
+        ct = cipher.encrypt(pt)
+
+        output = bytearray(16)
+        cipher = AES.new(b'4'*16, self.aes_mode, iv=self.iv_128)
+        res = cipher.encrypt(pt, output=output)
+        self.assertEqual(ct, output)
+        self.assertEqual(res, None)
+        
+        cipher = AES.new(b'4'*16, self.aes_mode, iv=self.iv_128)
+        res = cipher.decrypt(ct, output=output)
+        self.assertEqual(pt, output)
+        self.assertEqual(res, None)
+
+    def test_output_param_memoryview(self):
+        
+        pt = b'5' * 16
+        cipher = AES.new(b'4'*16, self.aes_mode, iv=self.iv_128)
+        ct = cipher.encrypt(pt)
+
+        output = memoryview(bytearray(16))
+        cipher = AES.new(b'4'*16, self.aes_mode, iv=self.iv_128)
+        cipher.encrypt(pt, output=output)
+        self.assertEqual(ct, output)
+        
+        cipher = AES.new(b'4'*16, self.aes_mode, iv=self.iv_128)
+        cipher.decrypt(ct, output=output)
+        self.assertEqual(pt, output)
+
+    def test_output_param_neg(self):
+
+        pt = b'5' * 16
+        cipher = AES.new(b'4'*16, self.aes_mode, iv=self.iv_128)
+        ct = cipher.encrypt(pt)
+
+        cipher = AES.new(b'4'*16, self.aes_mode, iv=self.iv_128)
+        self.assertRaises(TypeError, cipher.encrypt, pt, output=b'0'*16)
+        
+        cipher = AES.new(b'4'*16, self.aes_mode, iv=self.iv_128)
+        self.assertRaises(TypeError, cipher.decrypt, ct, output=b'0'*16)
+
+        shorter_output = bytearray(15)
+        cipher = AES.new(b'4'*16, self.aes_mode, iv=self.iv_128)
+        self.assertRaises(ValueError, cipher.encrypt, pt, output=shorter_output)
+        cipher = AES.new(b'4'*16, self.aes_mode, iv=self.iv_128)
+        self.assertRaises(ValueError, cipher.decrypt, ct, output=shorter_output)
+
+
+    import sys
+    if sys.version[:3] == "2.6":
         del test_memoryview
+        del test_output_param_memoryview
 
 
 class CbcTests(BlockChainingTests):
@@ -254,7 +307,7 @@ class NistBlockChainingVectors(unittest.TestCase):
         for tv in test_vectors:
 
             # The test vector file contains some directive lines
-            if isinstance(tv, basestring):
+            if is_string(tv):
                 direction = tv
                 continue
 
@@ -280,7 +333,7 @@ class NistBlockChainingVectors(unittest.TestCase):
         for tv in test_vectors:
 
             # The test vector file contains some directive lines
-            if isinstance(tv, basestring):
+            if is_string(tv):
                 direction = tv
                 continue
 
@@ -289,13 +342,13 @@ class NistBlockChainingVectors(unittest.TestCase):
 
             if direction == '[ENCRYPT]':
                 cts = [ tv.iv ]
-                for count in xrange(1000):
+                for count in range(1000):
                     cts.append(cipher.encrypt(tv.plaintext))
                     tv.plaintext = cts[-2]
                 self.assertEqual(cts[-1], tv.ciphertext)
             elif direction == '[DECRYPT]':
                 pts = [ tv.iv]
-                for count in xrange(1000):
+                for count in range(1000):
                     pts.append(cipher.decrypt(tv.ciphertext))
                     tv.ciphertext = pts[-2]
                 self.assertEqual(pts[-1], tv.plaintext)
@@ -313,7 +366,7 @@ class NistBlockChainingVectors(unittest.TestCase):
         for tv in test_vectors:
 
             # The test vector file contains some directive lines
-            if isinstance(tv, basestring):
+            if is_string(tv):
                 direction = tv
                 continue
 
